@@ -11,15 +11,16 @@ from numpy.linalg import svd
 from .basic_manifold_jax import basic_manifold_jax
 
 class hyperbolic_jax(basic_manifold_jax):
-    def __init__(self, var_shape, B = None, train_mode = False) -> None:
+    def __init__(self, var_shape, B = None, train_mode = False, device = None) -> None:
         n = var_shape[0]
         p = var_shape[1]
         self._n = n
         self._p = p
         self.dim = n*p 
+        
+        self.device = device
 
         self.var_shape = (n,p)
-
         self._half_n = int(n/2)
         
         if B is None:
@@ -49,8 +50,8 @@ class hyperbolic_jax(basic_manifold_jax):
 
         self.B = B 
         if not train_mode:
-            super().__init__('Hyperbolic',(n,p), (p,p))
-        self.Ip = jnp.eye(self._p)
+            super().__init__('Hyperbolic',(n,p), (p,p), device= self.device)
+        self.Ip = jax.device_put(jnp.eye(self._p), self.device)
         # Here we require B be a function
         #  when one wishes to input a matrix B_mat
         #  we recommand to use lambda X: B@X instead
@@ -126,6 +127,7 @@ class hyperbolic_jax(basic_manifold_jax):
                 # X = X + 0.00001/self._n * np.random.randn(self._n, self._p)
             if feas < 1e-8:
                 break
+        X = jax.device_put(X, self.device)
         return X
 
     def Post_process(self,X):
