@@ -142,7 +142,7 @@ class stiefel_jax(basic_manifold_jax):
 
             return (   jnp.matmul(local_objhess_JAT_D , (self.Ip - 0.5 * CX)) 
                     -  jnp.matmul(X , self.Phi( jnp.matmul(X.swapaxes(-2,-1) , local_objhess_JAT_D) + self.Phi(jnp.matmul(D.swapaxes(-2,-1) , gradf)) - 4* beta * XD) )
-                    + jnp.matmul(D , (2*beta*CX - XG) - jnp.matmul(gradf , XD)  )   )
+                    + jnp.matmul(D , (2*beta*CX - XG)) - jnp.matmul(gradf , XD)     )
 
 
 
