@@ -7,13 +7,13 @@ import numpy as np
 
 from ..manifold import basic_manifold
 class basic_manifold_np(basic_manifold):
-    def __init__(self,name,variable_shape, constraint_shape,  regularize_value = 0.01, safegurad_value = 0) -> None:
+    def __init__(self,name,variable_shape, constraint_shape,  regularize_value = 0.01, safeguard_value = 0) -> None:
         self.name = name
         self.shape = (variable_shape,) 
         self.var_shape = variable_shape
         self.con_shape = constraint_shape
         self.regularize_value = regularize_value
-        self.safegurad_value = safegurad_value
+        self.safeguard_value = safeguard_value
 
         
         # self.manifold_type = 'S'
@@ -23,7 +23,7 @@ class basic_manifold_np(basic_manifold):
         # self.Ip = torch.eye()
 
         
-        super().__init__(self.name,self.var_shape, self.con_shape,  regularize_value = self.regularize_value, safegurad_value = self.safegurad_value, backbone = 'autograd',)
+        super().__init__(self.name,self.var_shape, self.con_shape,  regularize_value = self.regularize_value, safeguard_value = self.safeguard_value, backbone = 'autograd',)
 
         
     
