@@ -6,4 +6,5 @@ from .utils import get_quad_penalty, wvt_flatten2d, wvt_flatten2d_transp, wvt_id
 __all__ = ["Linear_cdopt", "Bilinear_cdopt", "LazyLinear_cdopt",
  "Conv1d_cdopt", "Conv2d_cdopt", "Conv3d_cdopt",  "ConvTranspose1d_cdopt", "ConvTranspose2d_cdopt", "ConvTranspose3d_cdopt",
  "RNNBase_cdopt", "RNN_cdopt", "LSTM_cdopt", "GRU_cdopt",
-"RNNCell_cdopt", "LSTMCell_cdopt", "GRUCell_cdopt"]
\ No newline at end of file
+"RNNCell_cdopt", "LSTMCell_cdopt", "GRUCell_cdopt",
+"get_quad_penalty", "wvt_flatten2d", "wvt_flatten2d_transp", "wvt_identical"]
\ No newline at end of file
