@@ -86,7 +86,7 @@ class _ConvNd_cdopt(nn.Module):
                  bias: bool,
                  padding_mode: str,
                  device=None,
-                 dtype=None, manifold_class = euclidean_torch,  weight_var_transfer = wvt_conv_flatten, **kwargs) -> None:
+                 dtype=None, manifold_class = euclidean_torch, penalty_param = 0, weight_var_transfer = wvt_conv_flatten, manifold_args = {}) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         super(_ConvNd_cdopt, self).__init__()
         if in_channels % groups != 0:
@@ -116,6 +116,7 @@ class _ConvNd_cdopt(nn.Module):
         self.output_padding = output_padding
         self.groups = groups
         self.padding_mode = padding_mode
+        self.penalty_param = penalty_param
         # `_reversed_padding_repeated_twice` is the padding to be passed to
         # `F.pad` if needed (e.g., for non-zero padding types that are
         # implemented as two ops: padding + conv). `F.pad` accepts paddings in
@@ -156,9 +157,10 @@ class _ConvNd_cdopt(nn.Module):
                 
         self.weight_to_var, self.var_to_weight, self.var_shape = weight_var_transfer(self.weight.size())
 
-        self.manifold = manifold_class(self.var_shape, device=device, dtype= dtype, **kwargs)
+        self.manifold = manifold_class(self.var_shape, device=device, dtype= dtype, **manifold_args)
         self.A = lambda X_tensor: self.var_to_weight(self.manifold.A( self.weight_to_var(X_tensor) ))
         self.C = lambda X_tensor: self.manifold.C( self.weight_to_var(X_tensor) )
+        self.feasibility = lambda : torch.linalg.norm( self.C(self.weight).flatten() )
         self.quad_penalty = lambda : torch.sum( self.C(self.weight) **2 )
         for key, param in self.manifold._parameters.items():
             self._parameters[key] = param
@@ -217,7 +219,8 @@ class Conv1d_cdopt(_ConvNd_cdopt):
         bias: bool = True,
         padding_mode: str = 'zeros',  # TODO: refine this type
         device=None,
-        dtype=None, manifold_class = euclidean_torch,  weight_var_transfer = wvt_conv_flatten, **kwargs
+        dtype=None, manifold_class = euclidean_torch, penalty_param = 0,  
+        weight_var_transfer = wvt_conv_flatten, manifold_args = {}
     ) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         # we create new variables below to make mypy happy since kernel_size has
@@ -228,7 +231,9 @@ class Conv1d_cdopt(_ConvNd_cdopt):
         dilation_ = _single(dilation)
         super(Conv1d_cdopt, self).__init__(
             in_channels, out_channels, kernel_size_, stride_, padding_, dilation_,
-            False, _single(0), groups, bias, padding_mode, **factory_kwargs, manifold_class = manifold_class,  weight_var_transfer = weight_var_transfer, **kwargs)
+            False, _single(0), groups, bias, padding_mode, **factory_kwargs, 
+            manifold_class = manifold_class, penalty_param=penalty_param,  
+            weight_var_transfer = weight_var_transfer, manifold_args = manifold_args)
 
 
 
@@ -260,7 +265,8 @@ class Conv2d_cdopt(_ConvNd_cdopt):
         bias: bool = True,
         padding_mode: str = 'zeros',  # TODO: refine this type
         device=None,
-        dtype=None, manifold_class = euclidean_torch,  weight_var_transfer = wvt_conv_flatten, **kwargs
+        dtype=None, manifold_class = euclidean_torch, penalty_param = 0, 
+        weight_var_transfer = wvt_conv_flatten, manifold_args = {}
     ) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         kernel_size_ = _pair(kernel_size)
@@ -269,7 +275,9 @@ class Conv2d_cdopt(_ConvNd_cdopt):
         dilation_ = _pair(dilation)
         super(Conv2d_cdopt, self).__init__(
             in_channels, out_channels, kernel_size_, stride_, padding_, dilation_,
-            False, _pair(0), groups, bias, padding_mode, **factory_kwargs, manifold_class = manifold_class,  weight_var_transfer = weight_var_transfer, **kwargs)
+            False, _pair(0), groups, bias, padding_mode, **factory_kwargs, 
+            manifold_class = manifold_class, penalty_param = penalty_param,  
+            weight_var_transfer = weight_var_transfer, manifold_args = manifold_args)
 
     def _conv_forward(self, input: Tensor, weight: Tensor, bias: Optional[Tensor]):
         if self.padding_mode != 'zeros':
@@ -298,7 +306,8 @@ class Conv3d_cdopt(_ConvNd_cdopt):
         bias: bool = True,
         padding_mode: str = 'zeros',
         device=None,
-        dtype=None, manifold_class = euclidean_torch,  weight_var_transfer = wvt_conv_flatten, **kwargs
+        dtype=None, manifold_class = euclidean_torch, penalty_param = 0, 
+        weight_var_transfer = wvt_conv_flatten, manifold_args = {}
     ) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         kernel_size_ = _triple(kernel_size)
@@ -307,7 +316,9 @@ class Conv3d_cdopt(_ConvNd_cdopt):
         dilation_ = _triple(dilation)
         super(Conv3d_cdopt, self).__init__(
             in_channels, out_channels, kernel_size_, stride_, padding_, dilation_,
-            False, _triple(0), groups, bias, padding_mode, **factory_kwargs, manifold_class = manifold_class,  weight_var_transfer = weight_var_transfer, **kwargs)
+            False, _triple(0), groups, bias, padding_mode, **factory_kwargs, 
+            manifold_class = manifold_class, penalty_param = penalty_param,  
+            weight_var_transfer = weight_var_transfer, manifold_args = manifold_args)
 
     def _conv_forward(self, input: Tensor, weight: Tensor, bias: Optional[Tensor]):
         if self.padding_mode != "zeros":
@@ -336,7 +347,7 @@ class _ConvTransposeNd_cdopt(_ConvNd_cdopt):
     def __init__(self, in_channels, out_channels, kernel_size, stride,
                  padding, dilation, transposed, output_padding,
                  groups, bias, padding_mode, device=None, dtype=None, 
-                 manifold_class = euclidean_torch,  weight_var_transfer = wvt_conv_flatten_transp, **kwargs) -> None:
+                 manifold_class = euclidean_torch, penalty_param = 0,  weight_var_transfer = wvt_conv_flatten_transp, manifold_args = {}) -> None:
         if padding_mode != 'zeros':
             raise ValueError('Only "zeros" padding mode is supported for {}'.format(self.__class__.__name__))
 
@@ -344,7 +355,9 @@ class _ConvTransposeNd_cdopt(_ConvNd_cdopt):
         super(_ConvTransposeNd_cdopt, self).__init__(
             in_channels, out_channels, kernel_size, stride,
             padding, dilation, transposed, output_padding,
-            groups, bias, padding_mode, **factory_kwargs, manifold_class=manifold_class, weight_var_transfer=weight_var_transfer, **kwargs)
+            groups, bias, padding_mode, **factory_kwargs, 
+            manifold_class=manifold_class, penalty_param = penalty_param, 
+            weight_var_transfer=weight_var_transfer, manifold_args = manifold_args)
 
     # dilation being an optional parameter is for backwards
     # compatibility
@@ -407,7 +420,8 @@ class ConvTranspose1d_cdopt(_ConvTransposeNd_cdopt):
         dilation: _size_1_t = 1,
         padding_mode: str = 'zeros',
         device=None,
-        dtype=None, manifold_class = euclidean_torch,  weight_var_transfer = wvt_conv_flatten_transp, **kwargs
+        dtype=None, manifold_class = euclidean_torch, penalty_param = 0, 
+        weight_var_transfer = wvt_conv_flatten_transp, manifold_args = {}
     ) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         kernel_size = _single(kernel_size)
@@ -417,7 +431,9 @@ class ConvTranspose1d_cdopt(_ConvTransposeNd_cdopt):
         output_padding = _single(output_padding)
         super(ConvTranspose1d_cdopt, self).__init__(
             in_channels, out_channels, kernel_size, stride, padding, dilation,
-            True, output_padding, groups, bias, padding_mode, **factory_kwargs, manifold_class=manifold_class, weight_var_transfer=weight_var_transfer, **kwargs)
+            True, output_padding, groups, bias, padding_mode, **factory_kwargs, 
+            manifold_class=manifold_class, penalty_param = penalty_param, 
+            weight_var_transfer=weight_var_transfer, manifold_args = manifold_args)
 
     def forward(self, input: Tensor, output_size: Optional[List[int]] = None) -> Tensor:
         if self.padding_mode != 'zeros':
@@ -450,7 +466,8 @@ class ConvTranspose2d_cdopt(_ConvTransposeNd_cdopt):
         dilation: _size_2_t = 1,
         padding_mode: str = 'zeros',
         device=None,
-        dtype=None, manifold_class = euclidean_torch,  weight_var_transfer = wvt_conv_flatten_transp, **kwargs
+        dtype=None, manifold_class = euclidean_torch, penalty_param = 0, 
+        weight_var_transfer = wvt_conv_flatten_transp, manifold_args = {}
     ) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         kernel_size = _pair(kernel_size)
@@ -460,7 +477,9 @@ class ConvTranspose2d_cdopt(_ConvTransposeNd_cdopt):
         output_padding = _pair(output_padding)
         super(ConvTranspose2d_cdopt, self).__init__(
             in_channels, out_channels, kernel_size, stride, padding, dilation,
-            True, output_padding, groups, bias, padding_mode, **factory_kwargs, manifold_class=manifold_class, weight_var_transfer=weight_var_transfer, **kwargs)
+            True, output_padding, groups, bias, padding_mode, **factory_kwargs, 
+            manifold_class=manifold_class, penalty_param = penalty_param, 
+            weight_var_transfer=weight_var_transfer, manifold_args = manifold_args)
 
     def forward(self, input: Tensor, output_size: Optional[List[int]] = None) -> Tensor:
         if self.padding_mode != 'zeros':
@@ -494,7 +513,8 @@ class ConvTranspose3d_cdopt(_ConvTransposeNd_cdopt):
         dilation: _size_3_t = 1,
         padding_mode: str = 'zeros',
         device=None,
-        dtype=None, manifold_class = euclidean_torch,  weight_var_transfer = wvt_conv_flatten_transp, **kwargs
+        dtype=None, manifold_class = euclidean_torch, penalty_param = 0, 
+        weight_var_transfer = wvt_conv_flatten_transp, manifold_args = {}
     ) -> None:
         factory_kwargs = {'device': device, 'dtype': dtype}
         kernel_size = _triple(kernel_size)
@@ -504,7 +524,9 @@ class ConvTranspose3d_cdopt(_ConvTransposeNd_cdopt):
         output_padding = _triple(output_padding)
         super(ConvTranspose3d_cdopt, self).__init__(
             in_channels, out_channels, kernel_size, stride, padding, dilation,
-            True, output_padding, groups, bias, padding_mode, **factory_kwargs, manifold_class=manifold_class, weight_var_transfer=weight_var_transfer, **kwargs)
+            True, output_padding, groups, bias, padding_mode, **factory_kwargs, 
+            manifold_class=manifold_class, penalty_param = penalty_param, 
+            weight_var_transfer=weight_var_transfer, manifold_args= manifold_args)
 
     def forward(self, input: Tensor, output_size: Optional[List[int]] = None) -> Tensor:
         if self.padding_mode != 'zeros':
